@@ -8,14 +8,17 @@ import logging
 import copy
 import numpy
 import gc
+import glob
 
 # from pympler import summary, muppy
 
-from typing import Union, List
+from typing import Union, List, Iterable
 
-from obspy import Stream, UTCDateTime, Inventory
+from obspy import Stream, UTCDateTime, Inventory, Trace
 from matplotlib.figure import Figure
+from multiprocessing import Process, Lock
 from eqcorrscan import Tribe, Template, Party, Detection
+from eqcorrscan.utils.pre_processing import _prep_data_for_correlation
 
 from rt_eqcorrscan.streaming.streaming import _StreamingClient
 from rt_eqcorrscan.event_trigger.triggers import average_rate
@@ -53,17 +56,24 @@ class RealTimeTribe(Tribe):
     plotting_exclude_channels
         Channels to exclude from plotting
     """
+    # Management of detection multi-processing
     process_cores = 2
     _parallel_processing = True  # This seems unstable for subprocessing.
+    max_correlation_cores = None
+
+    # Thread management
+    lock = Lock()  # Lock for access to internals
     _running = False
     _detecting_thread = None
+    _backfillers = []  # Backfill processes
+    _number_of_backfillers = 0  # Book-keeping of backfiller processes.
     busy = False
-    _speed_up = 1.0
-    # Speed-up for simulated runs - do not change for real-time!
+
+    _speed_up = 1.0  # For simulated runs - do not change for real-time!
     _max_wait_length = 60.
-    _fig = None
-    _tribe_file = "tribe.tgz"
-    _min_run_length = 3600  # Minimum run-length in seconds.
+    _fig = None  # Cache figures to save memory
+    _template_dir = "new_templates"  # Where new templates should be.
+    _min_run_length = 24 * 3600  # Minimum run-length in seconds.
     # Usurped by max_run_length, used to set a threshold for rate calculation.
 
     def __init__(
@@ -82,7 +92,6 @@ class RealTimeTribe(Tribe):
             [template.process_length for template in self.templates]))
         assert (self.rt_client.buffer_capacity >= detect_interval)
         self.name = name or "RealTimeTribe"
-        self.buffer = Stream()
         self.inventory = inventory
         self.party = Party()
         self.detect_interval = detect_interval
@@ -172,10 +181,22 @@ class RealTimeTribe(Tribe):
             if d.detect_time <= endtime:
                 self.detections.remove(d)
 
+    def _remove_unused_backfillers(self):
+        """ Expire unused back fill processes to release resources. """
+        active_backfillers = []
+        for backfill_process in self._backfillers:
+            if not backfill_process.is_alive():
+                backfill_process.join()
+                backfill_process.close()
+            else:
+                active_backfillers.append(backfill_process)
+        self._backfillers = active_backfillers
+
     def _handle_detections(
         self,
         new_party: Party,
         trig_int: float,
+        hypocentral_separation: float,
         endtime: UTCDateTime,
         detect_directory: str,
         save_waveforms: bool,
@@ -191,6 +212,9 @@ class RealTimeTribe(Tribe):
             The party of new detections
         trig_int
             Minimum inter-detection time in seconds.
+        hypocentral_separation
+            Maximum inter-event distance in km to consider detections as being
+            duplicates.
         endtime
             Last detection-time to be kept.
         detect_directory
@@ -211,29 +235,33 @@ class RealTimeTribe(Tribe):
                 d._calculate_event(template=family.template)
             self.party += family
         Logger.info("Removing duplicate detections")
-        # TODO: Decluster on pick time? Find matching picks and calc median
-        #  pick time difference.
         if len(self.party) > 0:
             self.party.decluster(
-                trig_int=trig_int, timing="origin", metric="cor_sum")
+                trig_int=trig_int, timing="origin", metric="cor_sum",
+                hypocentral_separation=hypocentral_separation)
         Logger.info("Completed decluster")
-        for family in self.party:
-            family.detections = [
-                d for d in family.detections if d.detect_time >= endtime]
         Logger.info("Writing detections to disk")
-        for f in self.party:
-            for d in f:
-                if d in self.detections:
+        for family in self.party:
+            for detection in family:
+                if detection in self.detections:
                     continue
-                Logger.info(f"Writing detection: {d.detect_time}")
+                Logger.info(f"Writing detection: {detection.detect_time}")
                 self._fig = _write_detection(
-                    detection=d,
+                    detection=detection,
                     detect_directory=detect_directory,
                     save_waveform=save_waveforms,
                     plot_detection=plot_detections, stream=st,
                     fig=self._fig)
+        Logger.info("Expiring old detections")
+        for family in self.party:
+            Logger.debug(f"Checking for {family.template.name}")
+            family.detections = [
+                d for d in family.detections if d.detect_time >= endtime]
+            Logger.debug(f"Appending {len(family)} detections")
+            for detection in family:
                 # Need to append rather than create a new object
-                self.detections.append(d)
+                self.detections.append(detection)
+        return
 
     def _plot(self) -> None:  # pragma: no cover
         """ Plot the data as it comes in. """
@@ -262,11 +290,14 @@ class RealTimeTribe(Tribe):
         wait_length = 0.
         while True:
             tic = time.time()
+            # Check on backfillers
+            self._remove_unused_backfillers()
             # Wait until we have some data
             Logger.debug(
                 "Waiting for data, currently have {0} channels of {1} "
                 "expected channels".format(
-                    len(self.rt_client.buffer), len(self.expected_seed_ids)))
+                    len(self.rt_client.buffer_ids),
+                    len(self.expected_seed_ids)))
             if detection_kwargs:
                 self._add_templates_from_disk(**detection_kwargs)
             else:
@@ -277,7 +308,7 @@ class RealTimeTribe(Tribe):
             toc = time.time()
             wait_length += (toc - tic)
             if wait is None:
-                if len(self.rt_client.buffer) >= len(self.expected_seed_ids):
+                if len(self.rt_client.buffer_ids) >= len(self.expected_seed_ids):
                     break
                 if wait_length >= max_wait:
                     Logger.warning(
@@ -293,6 +324,7 @@ class RealTimeTribe(Tribe):
         threshold: float,
         threshold_type: str,
         trig_int: float,
+        hypocentral_separation: float = None,
         min_stations: int = None,
         keep_detections: float = 86400,
         detect_directory: str = "{name}/detections",
@@ -321,6 +353,9 @@ class RealTimeTribe(Tribe):
             `eqcorrscan.core.match_filter.Tribe.detect` for options.
         trig_int
             Minimum inter-detection time in seconds.
+        hypocentral_separation
+            Maximum inter-event distance in km to consider detections as being
+            duplicates.
         min_stations
             Minimum number of stations required to make a detection.
         keep_detections
@@ -352,6 +387,13 @@ class RealTimeTribe(Tribe):
         The party created - will not contain detections expired by
         `keep_detections` threshold.
         """
+        # Reshape the templates first
+        if len(self.templates) > 0:
+            self.templates = reshape_templates(
+                templates=self.templates, used_seed_ids=self.expected_seed_ids)
+        else:
+            Logger.error("No templates, will not run")
+            return
         # Remove templates that do not have enough stations in common with the
         # inventory.
         min_stations = min_stations or 0
@@ -361,11 +403,19 @@ class RealTimeTribe(Tribe):
         Logger.info(
             f"{n} templates were removed because they did not share enough "
             f"stations with the inventory. {len(self.templates)} will be used")
+        if len(self.templates) == 0:
+            Logger.critical("No templates remain, not running")
+            return Party()
+        # Fix unsupported args
         try:
             if kwargs.pop("plot"):
                 Logger.info("EQcorrscan plotting disabled")
         except KeyError:
             pass
+        _cores = kwargs.get("cores", None)
+        if _cores:
+            self.max_correlation_cores = min(
+                kwargs.pop("cores"), self.max_correlation_cores)
         run_start = UTCDateTime.now()
         detection_iteration = 0  # Counter for number of detection loops run
         if not self.busy:
@@ -391,24 +441,35 @@ class RealTimeTribe(Tribe):
         Logger.info("Detection will use the following data: {0}".format(
             self.expected_seed_ids))
         if backfill_client and backfill_to:
+            backfill = Stream()
             self._wait()
+            _buffer = self.rt_client.stream
             for tr_id in self.expected_seed_ids:
                 try:
-                    tr_in_buffer = self.rt_client.buffer.select(id=tr_id)[0]
+                    tr_in_buffer = _buffer.select(id=tr_id)[0]
                 except IndexError:
                     continue
                 endtime = tr_in_buffer.stats.starttime
+                if endtime - backfill_to > self.rt_client.buffer_capacity:
+                    Logger.info("Truncating backfill to buffer length")
+                    starttime = endtime - self.rt_client.buffer_capacity
+                else:
+                    starttime = backfill_to
                 try:
                     tr = backfill_client.get_waveforms(
-                        *tr_id.split('.'), starttime=backfill_to,
+                        *tr_id.split('.'), starttime=starttime,
                         endtime=endtime).merge()[0]
                 except Exception as e:
                     Logger.error("Could not back fill due to: {0}".format(e))
                     continue
                 Logger.debug("Downloaded backfill: {0}".format(tr))
+                backfill += tr
+            for tr in backfill:
+                # Get the lock!
+                Logger.debug(f"Adding {tr.id} to the buffer")
                 self.rt_client.on_data(tr)
-                Logger.debug("Stream in buffer is now: {0}".format(
-                    self.rt_client.buffer))
+            Logger.debug("Stream in buffer is now: {0}".format(
+                self.rt_client.stream))
         if self.plot:  # pragma: no cover
             # Set up plotting thread
             self._plot()
@@ -421,35 +482,46 @@ class RealTimeTribe(Tribe):
                 sleep_step))
             self._wait(sleep_step)
         first_data = min([tr.stats.starttime
-                          for tr in self.rt_client.get_stream().merge()])
+                          for tr in self.rt_client.stream.merge()])
         detection_kwargs = dict(
             threshold=threshold, threshold_type=threshold_type,
-            trig_int=trig_int, keep_detections=keep_detections,
-            detect_directory=detect_directory, plot_detections=plot_detections,
-            save_waveforms=save_waveforms, maximum_backfill=first_data,
-            endtime=None, min_stations=min_stations)
+            trig_int=trig_int, hypocentral_separation=hypocentral_separation,
+            keep_detections=keep_detections, detect_directory=detect_directory,
+            plot_detections=plot_detections, save_waveforms=save_waveforms,
+            maximum_backfill=first_data, endtime=None,
+            min_stations=min_stations)
         try:
             while self.busy:
                 self._running = True  # Lock tribe
                 start_time = UTCDateTime.now()
-                st = self.rt_client.get_stream().split().merge()
+                st = self.rt_client.stream.split().merge()
                 # Split to remove trailing mask
                 if len(st) == 0:
                     Logger.warning("No data")
                     continue
                 # Cope with data that doesn't come
                 last_data = max(tr.stats.endtime for tr in st)
+                # Remove any data that shouldn't be there - sometimes GeoNet's
+                # Seedlink client gives old data.
+                st.trim(
+                    starttime=last_data - (self.rt_client.buffer_capacity + 20.0),
+                    endtime=last_data)
                 if detection_iteration > 0:
                     # For the first run we want to detect in everything we have.
                     st.trim(
                         starttime=last_data - self.minimum_data_for_detection,
                         endtime=last_data)
+                if len(st) == 0:
+                    Logger.warning("No data")
+                    continue
                 # Remove short channels
                 st.traces = [
                     tr for tr in st
-                    if _numpy_len(tr.data) >= (.8 * self.minimum_data_for_detection)]
+                    if _numpy_len(tr.data) >= (
+                            .8 * self.minimum_data_for_detection)]
                 Logger.info("Starting detection run")
-                Logger.debug("Using data: \n{0}".format(st.__str__(extended=True)))
+                Logger.info("Using data: \n{0}".format(
+                    st.__str__(extended=True)))
                 try:
                     Logger.debug("Currently have {0} templates in tribe".format(
                         len(self)))
@@ -457,9 +529,11 @@ class RealTimeTribe(Tribe):
                         stream=st, plot=False, threshold=threshold,
                         threshold_type=threshold_type, trig_int=trig_int,
                         xcorr_func="fftw", concurrency="concurrent",
+                        cores=self.max_correlation_cores,
                         process_cores=self.process_cores,
                         parallel_process=self._parallel_processing,
                         ignore_bad_data=True, **kwargs)
+                    Logger.info("Completed detection")
                 except Exception as e:  # pragma: no cover
                     Logger.error(e)
                     Logger.error(traceback.format_exc())
@@ -472,15 +546,19 @@ class RealTimeTribe(Tribe):
                         "better".format(self.detect_interval))
                     time.sleep(self.detect_interval)
                     continue
-                self._handle_detections(
-                    new_party, trig_int=trig_int,
-                    endtime=last_data - keep_detections,
-                    detect_directory=detect_directory,
-                    save_waveforms=save_waveforms,
-                    plot_detections=plot_detections, st=st)
-                self._remove_old_detections(last_data - keep_detections)
-                Logger.info("Party now contains {0} detections".format(
-                    len(self.detections)))
+                Logger.info(f"Trying to get lock - lock status: {self.lock}")
+                with self.lock:
+                    if len(new_party) > 0:
+                        self._handle_detections(
+                            new_party, trig_int=trig_int,
+                            hypocentral_separation=hypocentral_separation,
+                            endtime=last_data - keep_detections,
+                            detect_directory=detect_directory,
+                            save_waveforms=save_waveforms,
+                            plot_detections=plot_detections, st=st)
+                    self._remove_old_detections(last_data - keep_detections)
+                    Logger.info("Party now contains {0} detections".format(
+                        len(self.detections)))
                 self._running = False  # Release lock
                 run_time = UTCDateTime.now() - start_time
                 Logger.info("Detection took {0:.2f}s".format(run_time))
@@ -521,11 +599,18 @@ class RealTimeTribe(Tribe):
         return self.party
 
     def _read_templates_from_disk(self):
-        if not os.path.isfile(self._tribe_file):
+        template_files = glob.glob(f"{self._template_dir}/*")
+        if len(template_files) == 0:
             return []
-        Logger.info(f"Checking for events in {self._tribe_file}")
-        new_tribe = Tribe().read(self._tribe_file)
-        os.remove(self._tribe_file)  # Remove file once done with it.
+        Logger.info(f"Checking for events in {self._template_dir}")
+        new_tribe = Tribe()
+        for template_file in template_files:
+            Logger.debug(f"Reading from {template_file}")
+            try:
+                new_tribe += Template().read(template_file)
+            except Exception as e:
+                Logger.error(f"Could not read {template_file} due to {e}")
+            os.remove(template_file)  # Remove file once done with it.
         new_tribe.templates = [t for t in new_tribe
                                if t.name not in self.running_templates]
         Logger.info(f"Read in {len(new_tribe)} new templates from disk")
@@ -541,7 +626,7 @@ class RealTimeTribe(Tribe):
         detect_directory: str = "{name}/detections",
         plot_detections: bool = True,
         save_waveforms: bool = True,
-        maximum_backfill: float = None,
+        maximum_backfill: Union[float, UTCDateTime] = None,
         endtime: UTCDateTime = None,
         **kwargs
     ):
@@ -568,7 +653,7 @@ class RealTimeTribe(Tribe):
         detect_directory: str = "{name}/detections",
         plot_detections: bool = True,
         save_waveforms: bool = True,
-        maximum_backfill: float = None,
+        maximum_backfill: Union[float, UTCDateTime] = None,
         endtime: UTCDateTime = None,
         **kwargs
     ) -> set:
@@ -578,6 +663,7 @@ class RealTimeTribe(Tribe):
         This method will run the new templates back in time, then append the
         templates to the already running tribe.
 
+        # TODO - make standard parameters and wrap methods to decorate them
         Parameters
         ----------
         templates
@@ -589,6 +675,8 @@ class RealTimeTribe(Tribe):
             `eqcorrscan.core.match_filter.Tribe.detect` for options.
         trig_int
             Minimum inter-detection time in seconds.
+        min_stations:
+            Minimum number of stations required to make a detection.
         keep_detections
             Duration to store detection in memory for in seconds.
         detect_directory
@@ -613,11 +701,15 @@ class RealTimeTribe(Tribe):
             Complete set of template names after addition
         """
         while self._running:
+            Logger.info("Waiting for access to tribe in add_templates")
             time.sleep(1)  # Wait until the lock is released
         if isinstance(templates, list):
             new_tribe = Tribe(templates)
         else:
             new_tribe = templates
+        # Reshape
+        new_tribe.templates = reshape_templates(
+            templates=new_tribe.templates, used_seed_ids=self.expected_seed_ids)
         # Remove templates that do not have enough stations.
         min_stations = min_stations or 0
         n = len(new_tribe)
@@ -630,36 +722,166 @@ class RealTimeTribe(Tribe):
             f"{n} templates were removed because they did not have enough "
             f"stations. {len(new_tribe)} will be added to the running tribe.")
         self.templates.extend(new_tribe.templates)
+        self.backfill(
+            templates=new_tribe, threshold=threshold,
+            threshold_type=threshold_type, trig_int=trig_int,
+            keep_detections=keep_detections, detect_directory=detect_directory,
+            plot_detections=plot_detections, save_waveforms=save_waveforms,
+            maximum_backfill=maximum_backfill, endtime=endtime, **kwargs)
+        return set(t.name for t in self.templates)
+
+    def backfill(
+        self,
+        templates: Union[List[Template], Tribe],
+        threshold: float,
+        threshold_type: str,
+        trig_int: float,
+        hypocentral_separation: float = None,
+        keep_detections: float = 86400,
+        detect_directory: str = "{name}/detections",
+        plot_detections: bool = True,
+        save_waveforms: bool = True,
+        maximum_backfill: Union[float, UTCDateTime] = None,
+        endtime: UTCDateTime = None,
+        **kwargs
+    ) -> None:
+        """
+        Backfill using data from rt_client's wavebank.
+
+        This method will run the new templates through old data and record
+        detections in the real-time-tribe.
+
+        Parameters
+        ----------
+        templates
+            New templates to add to the tribe.
+        threshold
+            Threshold for detection
+        threshold_type
+            Type of threshold to use. See
+            `eqcorrscan.core.match_filter.Tribe.detect` for options.
+        trig_int
+            Minimum inter-detection time in seconds.
+        hypocentral_separation
+            Maximum inter-event distance in km to consider detections as being
+            duplicates.
+        keep_detections
+            Duration to store detection in memory for in seconds.
+        detect_directory
+            Relative path to directory for detections. This directory will be
+            created if it doesn't exist - tribe name will be appended to this
+            string to give the directory name.
+        plot_detections
+            Whether to plot detections or not - plots will be saved to the
+            `detect_directory` as png images.
+        save_waveforms
+            Whether to save waveforms of detections or not - waveforms
+            will be saved in the `detect_directory` as miniseed files.
+        maximum_backfill
+            Time in seconds to backfill to - if this is larger than the
+            difference between the time now and the time that the tribe
+            started, then it will backfill to when the tribe started.
+        endtime
+            Time to stop the backfill, if None will run to now.
+        """
+        self._number_of_backfillers += 1
+        backfill_process = Process(
+            target=self._backfill,
+            args=(templates, threshold, threshold_type, trig_int,
+                  hypocentral_separation, keep_detections, detect_directory,
+                  plot_detections, save_waveforms, maximum_backfill, endtime),
+            kwargs=kwargs, name=f"Backfiller_{self._number_of_backfillers}")
+        backfill_process.start()
+        self._backfillers.append(backfill_process)
+
+    def _backfill(
+        self,
+        templates: Union[List[Template], Tribe],
+        threshold: float,
+        threshold_type: str,
+        trig_int: float,
+        hypocentral_separation: float = None,
+        keep_detections: float = 86400,
+        detect_directory: str = "{name}/detections",
+        plot_detections: bool = True,
+        save_waveforms: bool = True,
+        maximum_backfill: float = None,
+        endtime: UTCDateTime = None,
+        **kwargs
+    ) -> None:
+        """ Background backfill method """
+        if isinstance(templates, Tribe):
+            new_tribe = templates
+        else:
+            new_tribe = Tribe(templates)
+        Logger.info(f"Backfilling with {len(new_tribe)} templates")
         # Get the stream
         endtime = endtime or UTCDateTime.now()
         if maximum_backfill is not None:
-            starttime = endtime - maximum_backfill
+            if isinstance(maximum_backfill, (float, int)):
+                starttime = endtime - maximum_backfill
+            elif isinstance(maximum_backfill, UTCDateTime):
+                starttime = maximum_backfill
+            else:
+                Logger.warning(
+                    f"maximum_backfill is {type(maximum_backfill)}, not float "
+                    "or UTCDateTime, starting from 0")
+                starttime = UTCDateTime(0)
         else:
             starttime = UTCDateTime(0)
-        if starttime >= endtime or self.rt_client.wavebank is None:
+        Logger.info(f"Backfilling between {starttime} and {endtime}")
+        if starttime >= endtime or not self.rt_client.has_wavebank:
             return
-        bulk = [tuple(chan.split('.').extend([starttime, endtime]))
-                for chan in self.expected_seed_ids]
-        st = self.rt_client.wavebank.get_waveforms_bulk(bulk)
+        if self.expected_seed_ids and len(self.expected_seed_ids) > 0:
+            bulk = []
+            for chan in self.expected_seed_ids:
+                query = chan.split('.')
+                query.extend([starttime, endtime])
+                bulk.append(tuple(query))
+        else:
+            Logger.warning("No expected seed ids")
+            return
+        if len(bulk) == 0:
+            Logger.warning("No bulk")
+            return
+        Logger.info(f"Getting stations for backfill: {bulk}")
+        st = self.rt_client.get_wavebank_stream(bulk)
         Logger.debug("Additional templates to be run: \n{0} "
                      "templates".format(len(new_tribe)))
-        # TODO: This could be in a non-blocking Process of it's own
+
+        Logger.info("Starting backfill detection run with:")
+        Logger.info(st.__str__(extended=True))
         new_party = new_tribe.detect(
             stream=st, plot=False, threshold=threshold,
             threshold_type=threshold_type, trig_int=trig_int,
             xcorr_func="fftw", concurrency="concurrent",
+            cores=self.max_correlation_cores,
             parallel_process=self._parallel_processing,
             process_cores=self.process_cores, **kwargs)
-        while self._running:
-            time.sleep(1)  # Wait until lock is released to add detections
-        self._handle_detections(
-            new_party=new_party, detect_directory=detect_directory,
-            endtime=endtime - keep_detections, plot_detections=plot_detections,
-            save_waveforms=save_waveforms, st=st, trig_int=trig_int)
-        return set(t.name for t in self.templates)
+        detect_directory = detect_directory.format(name=self.name)
+        Logger.info("Backfill detection completed - handling detections")
+        if len(new_party) > 0:
+            Logger.info(f"Lock status: {self.lock}")
+            with self.lock:  # The only time the state is altered
+                Logger.info(f"Lock status: {self.lock}")
+                self._handle_detections(
+                    new_party=new_party, detect_directory=detect_directory,
+                    endtime=endtime - keep_detections,
+                    plot_detections=plot_detections,
+                    save_waveforms=save_waveforms, st=st, trig_int=trig_int,
+                    hypocentral_separation=hypocentral_separation)
+        return
 
-    def stop(self) -> None:
-        """ Stop the real-time system. """
+    def stop(self, write_stopfile: bool = False) -> None:
+        """
+        Stop the real-time system.
+       
+        Parameters
+        ----------
+        write_stopfile:
+            Used to write a one-line file telling listening systems that
+            this has stopped. Used by the Reactor.
+        """
         if self.plotter is not None:  # pragma: no cover
             self.plotter.background_stop()
         self.rt_client.background_stop()
@@ -667,6 +889,58 @@ class RealTimeTribe(Tribe):
         self._running = False
         if self._detecting_thread is not None:
             self._detecting_thread.join()
+        if write_stopfile:
+            with open(".stopfile", "a") as f:
+                f.write(f"{self.name}\n")
+
+
+def reshape_templates(
+    templates: List[Template],
+    used_seed_ids: Iterable[str]
+) -> List[Template]:
+    """
+    Reshape templates to have the full set of required channels (and no more).
+
+    This is done within matched-filter as well as here - we do it here so that
+    the templates are not reshaped every iteration.
+
+    Parameters
+    ----------
+    templates
+        Templates to be reshaped - the templates are changed in-place
+    used_seed_ids
+        Seed-ids used for detection (network.station.location.channel)
+
+    Returns
+    -------
+    Templates filled for detection.
+    """
+    template_streams = [t.st for t in templates]
+    template_names = [t.name for t in templates]
+
+    samp_rate = template_streams[0][0].stats.sampling_rate
+    process_len = max(t.process_length for t in templates)
+    # Make a dummy stream with all the used seed ids
+    stream = Stream()
+    for seed_id in used_seed_ids:
+        net, sta, loc, chan = seed_id.split('.')
+        tr = Trace(header=dict(
+            network=net, station=sta, location=loc, channel=chan,
+            sampling_rate=samp_rate),
+            data=numpy.empty(int(process_len * samp_rate)))
+        stream += tr
+
+    _, template_streams, template_names = _prep_data_for_correlation(
+        stream=stream, templates=template_streams,
+        template_names=template_names, force_stream_epoch=False)
+    templates_back = []
+    for template_st, template_name in zip(template_streams, template_names):
+        original = [t for t in templates if t.name == template_name]
+        assert len(original) == 1
+        original = original[0]
+        original.st = template_st
+        templates_back.append(original)
+    return templates_back
 
 
 def _write_detection(
